package org.zeroturnaround.javarebel.integration.fileservlet;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtField;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.LoggerFactory;
import org.zeroturnaround.javarebel.RebelServletConextFacade;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>org.apache.catalina.servlets.DefaultServlet</code>
 * or <code>org.mortbay.jetty.servlet.Default</code>
 * or <code>org.mortbay.jetty.servlet.DefaultServlet</code>.
 * 
 * @see FileServlet
 * 
 * @author Rein Raudjärv
 */
public class DefaultServletCBP extends JavassistClassBytecodeProcessor {

  private static final String REBEL_FIELD_TYPE = "org.zeroturnaround.javarebel.integration.fileservlet.FileServlet";
  private static final String REBEL_FIELD_NAME = "__rebel_default_servlet";
  
  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("org.zeroturnaround.javarebel");
    
    ctClass.addField(CtField.make("private " + REBEL_FIELD_TYPE + " " + REBEL_FIELD_NAME + ";", ctClass));
    
    try {
      ctClass.getDeclaredMethod("init").insertAfter(
          "{" +
          "  try {" +
          "    ServletIntegration integration = ServletIntegrationFactory.getInstance();" +
          "    RebelServletContext sc = (RebelServletContext) getServletContext();" +
          "    if (sc instanceof " + RebelServletConextFacade.class.getName() + ")" +
          "      sc = ((" + RebelServletConextFacade.class.getName() + ") sc)." + RebelServletConextFacade.METHOD_NAME + "();" +

          "    boolean hasReplacedResources = integration.hasReplacedResources(sc);" +
          "    LoggerFactory.getInstance().log($0.getClass().getName() + \"@\" + System.identityHashCode($0) + \": \" +" +
          "      \"Servlet Context \" + sc.getClass().getName() + \"@\" + System.identityHashCode(sc) +" +
          "      \" has replaced resources: \" + hasReplacedResources);" +

          "    if (hasReplacedResources) {" +
          "      " + REBEL_FIELD_NAME + " = new " + REBEL_FIELD_TYPE + "();" +
          "      " + REBEL_FIELD_NAME + ".init(getServletConfig());" +
          "    }" +
          "  } catch (Throwable t) {" +
          "    LoggerFactory.getInstance().error(t);" +
          "  }" +
          "}");
    } catch (NotFoundException e) {
    }
    
    try {
      ctClass.getDeclaredMethod("destroy").insertBefore(
          "{" +
          "  try {" +
          "    if (" + REBEL_FIELD_NAME + " != null) {" +
          "      " + REBEL_FIELD_NAME + ".destroy();" +
          "    }" +
          "  } catch (Throwable t) {" +
          "    LoggerFactory.getInstance().error(t);" +
          "  }" +
          "}");
    } catch (NotFoundException e) {
    }
    
    try {
      ctClass.getDeclaredMethod("service").insertBefore(
          "{" +
          "  if (" + REBEL_FIELD_NAME + " != null) {" +
          "    " + REBEL_FIELD_NAME + ".service($1, $2);" +
          "    return;" +
          "  }" +
      "}");
    } catch (NotFoundException e) {
    }
    
    try {
      ctClass.getDeclaredMethod("doGet").insertBefore(
          "{" +
          "  if (" + REBEL_FIELD_NAME + " != null) {" +
          "    " + REBEL_FIELD_NAME + ".doGet($1, $2);" +
          "    return;" +
          "  }" +
          "}");
    } catch (NotFoundException e) {
    }    
    
    try {
      ctClass.getDeclaredMethod("doPost").insertBefore(
          "{" +
          "  if (" + REBEL_FIELD_NAME + " != null) {" +
          "    " + REBEL_FIELD_NAME + ".doPost($1, $2);" +
          "    return;" +
          "  }" +
      "}");
    } catch (NotFoundException e) {
    }    
    
    
  }
}
