package org.zeroturnaround.javarebel.integration.fileservlet;

import java.util.ArrayList;
import java.util.List;
import org.zeroturnaround.javarebel.ClassResourceSource;
import org.zeroturnaround.javarebel.IntegrationFactory;
import org.zeroturnaround.javarebel.Plugin;


/**
 * JRebel plugin for file servlets.
 * <p>
 * This plugin enables JRebel to handle HTTP requests for static resources.
 * This is required for packaged development only.
 * </p>
 * <p>
 * The HTTP requests are handled by
 * <code>org.zeroturnaround.javarebel.integration.fileservlet.FileServlet</code>.
 * 
 * @author Rein Raudjärv
 */
public class FileServletPlugin implements Plugin {

  private static final String[] classNames;
  
  static {
    List result = new ArrayList();
      
    // JETTY 5.x
    result.add("org.mortbay.jetty.servlet.Default");
    // JETTY 6.x (Tested on 6.1.12)
    result.add("org.mortbay.jetty.servlet.DefaultServlet");
    
    /*
     * TOMCAT 4.1, 5.5, 6.0 (Tested on 4.1.37, 5.5.27, 6.0.14)
     * GLASSFISH 2.1 (Tested on 2.1-b60e)
     * JBOSS 4.2, 5.0 (Tested on 4.2.2 GA, 5.0.0 GA)
     */   
    result.add("org.apache.catalina.servlets.DefaultServlet");
    
    // CAUCHO RESIN (Tested on 3.0.24)
    result.add("com.caucho.servlets.FileServlet");

    // SAP NetWeaver 7.0
    result.add("com.sap.engine.services.servlets_jsp.server.servlet.DefaultServlet");
    
    classNames = (String[]) result.toArray(new String[result.size()]);
  }
  
  public void preinit() {
    IntegrationFactory.getInstance().addIntegrationProcessor(
        getClass().getClassLoader(), classNames, new DefaultServletCBP());
  }

  public boolean checkDependencies(ClassLoader cl, ClassResourceSource crs) {
    for (int i = 0; i < classNames.length; i++) {
      String className = classNames[i];
      if (crs.getClassResource(className) != null)
        return true;
    }
    return false;
  }

  public String getId() {
    return "fileservlet_plugin";
  }

  public String getName() {
    return null;
  }

  public String getDescription() {
    return null;
  }

  public String getAuthor() {
    return null;
  }

  public String getWebsite() {
    return null;
  }  
}
